/*
	pyLAPJV by Harold Cooper (hbc@mit.edu)
	pyLAPJV.cpp 2004-08-20
*/

#include "lap.h"

static PyObject *
LAPJV_lap(PyObject *self, PyObject *args)
//lap(costs)
{
    PyObject *ocosts;
	PyArrayObject *costs;
	int n;
	col *rowsol;
	row *colsol;
	cost lapcost,*buf,**ccosts,*u,*v;

    if (!PyArg_ParseTuple(args, "O", &ocosts))
        return NULL;
	costs = NA_InputArray(ocosts,COST_TYPE,NUM_C_ARRAY);
	if (costs->nd!=2)
	{
		PyErr_SetString(PyExc_ValueError,"lap() requires a 2-D matrix");
		goto error;
	}
	n = costs->dimensions[0];
	if (costs->dimensions[1]!=n)
	{
		PyErr_SetString(PyExc_ValueError,"lap() requires a square matrix");
		goto error;
	}
		
	//get inputted matrix as a 1-D C array:
	buf = (cost *)NA_OFFSETDATA(costs);
	
	//copy inputted matrix into a 2-dimensional C array:
	ccosts = (cost **)malloc(sizeof(cost *)*n);
	if(!ccosts)
	{
		PyErr_NoMemory();
		goto error;
	}
	for(int i=0;i<n;i++)
		ccosts[i] = buf+i*n;
		
	rowsol = (col *)malloc(sizeof(col)*n);
	colsol = (row *)malloc(sizeof(row)*n);
	u = (cost *)malloc(sizeof(cost)*n);
	v = (cost *)malloc(sizeof(cost)*n);
	if(!(rowsol&&colsol&&u&&v))
	{
		PyErr_NoMemory();
		goto error;
	}
	
	//run LAPJV!:
	lapcost = lap(n,ccosts,rowsol,colsol,u,v);
	
	//NA_InputArray() incremented costs, but now we're done with it, so let it get GC'ed:
	Py_XDECREF(costs);
	
	return Py_BuildValue("(dOOOO)",lapcost,
		NA_NewArray(rowsol,COL_TYPE,1,n),NA_NewArray(colsol,ROW_TYPE,1,n),
		NA_NewArray(u,COST_TYPE,1,n),NA_NewArray(v,COST_TYPE,1,n));
	
error:
	Py_XDECREF(costs);
	return NULL;
}

static PyMethodDef LAPJVMethods[] = {
    {"lap",  LAPJV_lap, METH_VARARGS,
     "Solves the linear assignment problem using the Jonker-Volgenant\nalgorithm.\n\nlap() takes a single argument - a square cost matrix - and returns a\n5-tuple of the form\n(min_cost,row_assigns,col_assigns,row_dual_vars,col_dual_vars).\n\nThe average user probably just wants the second or third of these\nelements, so a call like: \'assigns=lap(costs)[2]\' would be the\nmost common use."},
    {NULL, NULL, 0, NULL}        /* Sentinel (terminates structure) */
};

PyMODINIT_FUNC
initLAPJV(void)
{
    (void) Py_InitModule("LAPJV", LAPJVMethods);
	import_libnumarray();
}

int
main(int argc, char *argv[])
{
    /* Pass argv[0] to the Python interpreter */
    Py_SetProgramName(argv[0]);

    /* Initialize the Python interpreter.  Required. */
    Py_Initialize();

    /* Add a static module */
    initLAPJV();
	
	return 0;
}
